@@ -36,6 +36,10 @@ DEFINE_TEST(test_option_r)
 	size_t s, buff_size_rounded;
 	int r, i;
 
+	buff = NULL;
+	p0 = NULL;
+	p1 = NULL;
+
 	/* Create an archive with one file. */
 	assertMakeFile("f1", 0644, "abc");
 	r = systemf("%s cf archive.tar --format=ustar f1 >step1.out 2>step1.err", testprog);
@@ -47,11 +51,9 @@ DEFINE_TEST(test_option_r)
 	/* Do some basic validation of the constructed archive. */
 	p0 = slurpfile(&s, "archive.tar");
 	if (!assert(p0 != NULL))
-		return;
-	if (!assert(s >= 2048)) {
-		free(p0);
-		return;
-	}
+		goto done;
+	if (!assert(s >= 2048))
+		goto done;
 	assertEqualMem(p0 + 0, "f1", 3);
 	assertEqualMem(p0 + 512, "abc", 3);
 	assertEqualMem(p0 + 1024, "\0\0\0\0\0\0\0\0", 8);
@@ -60,10 +62,8 @@ DEFINE_TEST(test_option_r)
 	/* Edit that file with a lot more data and update the archive with a new copy. */
 	buff = malloc(buff_size);
 	assert(buff != NULL);
-	if (buff == NULL) {
-		free(p0);
-		return;
-	}
+	if (buff == NULL)
+		goto done;
 
 	for (i = 0; i < (int)buff_size; ++i)
 		buff[i] = "abcdefghijklmnopqrstuvwxyz"[rand() % 26];
@@ -77,10 +77,8 @@ DEFINE_TEST(test_option_r)
 
 	/* The constructed archive should just have the new entry appended. */
 	p1 = slurpfile(&s, "archive.tar");
-	if (!assert(p1 != NULL)) {
-		free(p0);
-		return;
-	}
+	if (!assert(p1 != NULL))
+		goto done;
 	buff_size_rounded = ((buff_size + 511) / 512) * 512;
 	assert(s >= 2560 + buff_size_rounded);
 	/* Verify first entry is unchanged. */
@@ -105,10 +103,8 @@ DEFINE_TEST(test_option_r)
 
 	/* Validate the constructed archive. */
 	p1 = slurpfile(&s, "archive.tar");
-	if (!assert(p1 != NULL)) {
-		free(p0);
-		return;
-	}
+	if (!assert(p1 != NULL))
+		goto done;
 	assert(s >= 3584 + buff_size_rounded);
 	/* Verify first two entries are unchanged. */
 	assertEqualMem(p0, p1, 1536 + buff_size_rounded);
@@ -118,7 +114,6 @@ DEFINE_TEST(test_option_r)
 	/* Verify end-of-archive marker. */
 	assertEqualMem(p1 + 2560 + buff_size_rounded, "\0\0\0\0\0\0\0\0", 8);
 	assertEqualMem(p1 + 3072 + buff_size_rounded, "\0\0\0\0\0\0\0\0", 8);
-	free(p0);
 	free(p1);
 
 	/* Unpack everything */
@@ -132,4 +127,7 @@ DEFINE_TEST(test_option_r)
 
 	/* Verify that the second copy of f1 overwrote the first. */
 	assertFileContents(buff, (int)strlen(buff), "f1");
+done:
+	free(buff);
+	free(p0);
 }
