@@ -25,55 +25,54 @@
 #include "test.h"
 __FBSDID("$FreeBSD: src/usr.bin/tar/test/test_strip_components.c,v 1.2 2008/11/10 05:24:13 kientzle Exp $");
 
-static int
-touch(const char *fn)
-{
-	int fd = open(fn, O_RDWR | O_CREAT, 0644);
-	failure("Couldn't create file '%s', fd=%d, errno=%d (%s)\n",
-	    fn, fd, errno, strerror(errno));
-	if (!assert(fd > 0))
-		return (0); /* Failure. */
-	close(fd);
-	return (1); /* Success */
-}
-
 DEFINE_TEST(test_strip_components)
 {
-	struct stat st;
+	assertMakeDir("d0", 0755);
+	assertChdir("d0");
+	assertMakeDir("d1", 0755);
+	assertMakeDir("d1/d2", 0755);
+	assertMakeDir("d1/d2/d3", 0755);
+	assertMakeFile("d1/d2/f1", 0644, "");
+	assertMakeHardlink("l1", "d1/d2/f1");
+	assertMakeHardlink("d1/l2", "d1/d2/f1");
+	if (canSymlink()) {
+		assertMakeSymlink("s1", "d1/d2/f1");
+		assertMakeSymlink("d1/s2", "d2/f1");
+	}
+	assertChdir("..");
 
-	assertEqualInt(0, mkdir("d0", 0755));
-	assertEqualInt(0, chdir("d0"));
-	assertEqualInt(0, mkdir("d1", 0755));
-	assertEqualInt(0, mkdir("d1/d2", 0755));
-	assertEqualInt(0, mkdir("d1/d2/d3", 0755));
-	assertEqualInt(1, touch("d1/d2/f1"));
-	assertEqualInt(0, link("d1/d2/f1", "l1"));
-	assertEqualInt(0, link("d1/d2/f1", "d1/l2"));
-	assertEqualInt(0, symlink("d1/d2/f1", "s1"));
-	assertEqualInt(0, symlink("d2/f1", "d1/s2"));
-	assertEqualInt(0, chdir(".."));
+	/*
+	 * Test 1: Strip components when extracting archives.
+	 */
+	if (canSymlink())
+		assertEqualInt(0, systemf("%s -cf test.tar d0/l1 d0/s1 d0/d1",
+		    testprog));
+	else
+		assertEqualInt(0, systemf("%s -cf test.tar d0/l1 d0/d1",
+		    testprog));
 
-	assertEqualInt(0, systemf("%s -cf test.tar d0", testprog));
-
-	assertEqualInt(0, mkdir("target", 0755));
+	assertMakeDir("target", 0755);
 	assertEqualInt(0, systemf("%s -x -C target --strip-components 2 "
 	    "-f test.tar", testprog));
 
 	failure("d0/ is too short and should not get restored");
-	assertEqualInt(-1, lstat("target/d0", &st));
+	assertFileNotExists("target/d0");
 	failure("d0/d1/ is too short and should not get restored");
-	assertEqualInt(-1, lstat("target/d1", &st));
+	assertFileNotExists("target/d1");
+	failure("d0/s1 is too short and should not get restored");
+	assertFileNotExists("target/s1");
 	failure("d0/d1/s2 is a symlink to something that won't be extracted");
-#if !defined(_WIN32) || defined(__CYGWIN__)
-	assertEqualInt(-1, stat("target/s2", &st));
-#else
-	skipping("symlink with stat()");
-#endif
-	assertEqualInt(0, lstat("target/s2", &st));
+	/* If platform supports symlinks, target/s2 is a broken symlink. */
+	/* If platform does not support symlink, target/s2 doesn't exist. */
+	assertFileNotExists("target/s2");
+	if (canSymlink())
+		assertIsSymlink("target/s2", "d2/f1");
 	failure("d0/d1/d2 should be extracted");
-	assertEqualInt(0, lstat("target/d2", &st));
+	assertIsDir("target/d2", -1);
 
 	/*
+	 * Test 1b: Strip components extracting archives involving hardlinks.
+	 *
 	 * This next is a complicated case.  d0/l1, d0/d1/l2, and
 	 * d0/d1/d2/f1 are all hardlinks to the same file; d0/l1 can't
 	 * be extracted with --strip-components=2 and the other two
@@ -83,18 +82,11 @@ DEFINE_TEST(test_strip_components)
 	 * which these three names get archived.  If d0/l1 is first,
 	 * none of the three can be restored.  If either of the longer
 	 * names are first, then the two longer ones can both be
-	 * restored.
+	 * restored.  Note that the "tar -cf" command above explicitly
+	 * lists d0/l1 before d0/d1.
 	 *
-	 * The tree-walking code used by bsdtar always visits files
-	 * before subdirectories, so bsdtar's behavior is fortunately
-	 * deterministic:  d0/l1 will always get stored first and the
-	 * other two will be stored as hardlinks to d0/l1.  Since
-	 * d0/l1 can't be extracted, none of these three will be
-	 * extracted.
-	 *
-	 * It may be worth extending this test to force a particular
-	 * archiving order so as to exercise both of the cases described
-	 * above.
+	 * It may be worth extending this test to exercise other
+	 * archiving orders.
 	 *
 	 * Of course, this is all totally different for cpio and newc
 	 * formats because the hardlink management is different.
@@ -102,9 +94,46 @@ DEFINE_TEST(test_strip_components)
 	 * parallel tests for cpio and newc formats.
 	 */
 	failure("d0/l1 is too short and should not get restored");
-	assertEqualInt(-1, lstat("target/l1", &st));
+	assertFileNotExists("target/l1");
 	failure("d0/d1/l2 is a hardlink to file whose name was too short");
-	assertEqualInt(-1, lstat("target/l2", &st));
+	assertFileNotExists("target/l2");
 	failure("d0/d1/d2/f1 is a hardlink to file whose name was too short");
-	assertEqualInt(-1, lstat("target/d2/f1", &st));
+	assertFileNotExists("target/d2/f1");
+
+	/*
+	 * Test 2: Strip components when creating archives.
+	 */
+	if (canSymlink())
+		assertEqualInt(0, systemf("%s --strip-components 2 -cf test2.tar "
+					"d0/l1 d0/s1 d0/d1", testprog));
+	else
+		assertEqualInt(0, systemf("%s --strip-components 2 -cf test2.tar "
+					"d0/l1 d0/d1", testprog));
+
+	assertMakeDir("target2", 0755);
+	assertEqualInt(0, systemf("%s -x -C target2 -f test2.tar", testprog));
+
+	failure("d0/ is too short and should not have been archived");
+	assertFileNotExists("target2/d0");
+	failure("d0/d1/ is too short and should not have been archived");
+	assertFileNotExists("target2/d1");
+	failure("d0/s1 is too short and should not get restored");
+	assertFileNotExists("target/s1");
+	/* If platform supports symlinks, target/s2 is included. */
+	if (canSymlink()) {
+		failure("d0/d1/s2 is a symlink to something included in archive");
+		assertIsSymlink("target2/s2", "d2/f1");
+	}
+	failure("d0/d1/d2 should be archived");
+	assertIsDir("target2/d2", -1);
+
+	/*
+	 * Test 2b: Strip components creating archives involving hardlinks.
+	 */
+	failure("d0/l1 is too short and should not have been archived");
+	assertFileNotExists("target/l1");
+	failure("d0/d1/l2 is a hardlink to file whose name was too short");
+	assertFileNotExists("target/l2");
+	failure("d0/d1/d2/f1 is a hardlink to file whose name was too short");
+	assertFileNotExists("target/d2/f1");
 }
